//===============================================================================
// PJM Interconnection, LLC. Sprego Application
// Comments provided by Glenn Long
//===============================================================================


namespace UIComposition.Views.SpinningReserveCasesView

{
    using System.Windows.Controls; 
    using System.Windows.Threading;
    using System.Collections.Generic;
    using System;
    using PJMBaseLibrary.Utilities.Logger;
    using PJMBaseLibrary.Utilities.ApplicationEnums;
    using System.ComponentModel;
    using UIComposition.Services;
    using UIComposition.Controllers;
    using System.Linq;
    using System.Collections.Specialized;
    using PJMBaseLibrary.BusinessObjectsBase;
    using PJMBaseLibrary.Utilities.CustomBinarySerializer;
    using UIComposition.Interfaces;
    using Microsoft.Practices.Unity;
    using System.Windows;
    /// <summary>
    /// Interaction logic for SpregoView.xaml
    /// </summary>
    public partial class SpinningReserveCasesListView : UserControl, ISpinningReserveCasesListView, IRefreshView
    {
        private int index = 0;
        private const string SAVE_REMINDER = "There were changes made to the data. Click Save All to save your changes.";
 
        public SpinningReserveCasesListView()
        {
            // This is required to load the view
            InitializeComponent();
        }

        public SpinningReserveCasesPresentationModel Model
        {
            get { return this.DataContext as SpinningReserveCasesPresentationModel; }
            //set { this.DataContext = value; }

            set
            {

                this.Dispatcher.BeginInvoke(
            DispatcherPriority.Normal,
            (DispatcherOperationCallback)delegate(object arg)
            {
                // HANDLED ON UI THREAD
                // Add collection to Model to provide de-select option
                //new SpinningReserveCasesPresentationModel(value.AreaUnitDetail);

                this.ControlAreaList.DataContext = value;
                this.AreaUnitDetails.DataContext = value;

                if (value.AreaUnitDetail != null)
                    this.AreaUnitDetails.DataSource = value.AreaUnitDetail.UnitDetailList;

                if (value.ControlZoneArea != null)
                    this.ControlAreaList.DataSource = value.ControlZoneArea.AreaList;   
                return null;
            },
            null);
            }
        }

        public void AutoRefreshCurrentView()
        {
            this.DataContext = null;
            IUnityContainer aContainer = ConnectionService.Instance.globalContainer;
            if (aContainer != null)
            {
                ISpinningReserveCasesListPresenter viewPresenter = aContainer.Resolve<ISpinningReserveCasesListPresenter>();

                viewPresenter.SetDetail(Convert.ToDecimal(CasesService.CaseID));
                this.DataContext = viewPresenter.View.Model;
            }
        }

       

        private void ControlAreaList_AssigningFieldLayoutToItem(object sender, Infragistics.Windows.DataPresenter.Events.AssigningFieldLayoutToItemEventArgs e)
        {
            e.FieldLayout = this.ControlAreaList.FieldLayouts[index];
            index++;

            if (index > 0) { index = 0; }
        }

        private void AreaUnitDetails_AssigningFieldLayoutToItem(object sender, Infragistics.Windows.DataPresenter.Events.AssigningFieldLayoutToItemEventArgs e)
        {

            e.FieldLayout = this.AreaUnitDetails.FieldLayouts[index];
            index++;

            if (index > 0) { index = 0; }

        }
        private void TabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.Dispatcher.BeginInvoke(
                DispatcherPriority.Loaded,
                (DispatcherOperationCallback)delegate(object arg)
                { SaveMsgReminder(); return null; }, null);


        }
        private void SaveMsgReminder()
        {
            if (SaveController.Instance.IsUpdated)
            {
                //SaveController.Instance.OnClientMessageDisplay(SAVE_REMINDER);
            }
        }
      

        private void AreaUnitDetails_RecordExpanded(object sender, Infragistics.Windows.DataPresenter.Events.RecordExpandedEventArgs e)
        {
       
            // do you have any kids?
            if (e.Record.HasChildren == true)
            {
                // kool, this gives me all of the child records under my expanded list - wow!
                var myChildRecords = e.Record.ViewableChildRecords.ToList();

                // Create an object of my type
                ASEngine.BusinessObjects.Entities.UnitDetails myManyCollection = new ASEngine.BusinessObjects.Entities.UnitDetails();

                foreach (Infragistics.Windows.DataPresenter.Record rec in myChildRecords)
                {
                    // Add my converted type to my master collection
                    myManyCollection.Add((ASEngine.BusinessObjects.Entities.UnitDetail)((Infragistics.Windows.DataPresenter.DataRecord)(rec)).DataItem);
                }
                // Set the collection to null first, here comes a new one - no don't do this it forces the timer to take over. just overwrite existing
                ListExpandedDeselectedResourcesList refreshExpandedList = new ListExpandedDeselectedResourcesList(myManyCollection);

            }


        }  
 
    }

   

    public class SpinningReserveDislpayHelper : INotifyPropertyChanged
    {
        public int Index { get; set; }
        ASEngine.BusinessObjects.Entities.UnitDetails _units;
        public ASEngine.BusinessObjects.Entities.UnitDetails UnitDetailList { get { return _units; } }

        public static ASEngine.BusinessObjects.Entities.UnitDetails _expunits;
        public static ASEngine.BusinessObjects.Entities.UnitDetails ExpandedResourcesList { get { return _expunits; } }
       
       
        public SpinningReserveDislpayHelper(ASEngine.BusinessObjects.Entities.UnitDetails units, string source)
        {
            if (source == "ParentSource")
            {
                _units = units;
            }
            else
            {
                // Store the group of records to be updated if the user groups the items
                // We need to get the unique id's of each record and update them against the master.
                _expunits = units;
            }

            if (UnitDetailList != null)
            {

                foreach (ASEngine.BusinessObjects.Entities.UnitDetail unit in UnitDetailList)
                    unit.PropertyChanged += (sender, e) =>
                {

                    // Save Modified
                    ASEngine.BusinessObjects.Entities.UnitDetail o = (ASEngine.BusinessObjects.Entities.UnitDetail)sender;
                    o.IsDirty = true;
                    //send update.
                    UpdateBO(o);

                    // Raising PropertyChanged for the AllUnitsAreChecked
                    // property causes the data binding system to query its
                    // getter for the new value.
                    if (e.PropertyName == "Tier1Deselected")
                    {
                        this.OnPropertyChanged("AllT1UnitsAreChecked");

                    }
                    else if (e.PropertyName == "Tier2Deselected")
                    {
                        this.OnPropertyChanged("AllT2UnitsAreChecked");
                    }
                    else if (e.PropertyName == "RegDeselected")
                    {
                        this.OnPropertyChanged("AllRegDeSelectAreChecked");
                    }

                };                                   

            }

            this.Index = 0;

        }


        public decimal? AllT1UnitsAreChecked
        {
            get
            {
                try
                {
                    // Determine if all units have the same 
                    // value for the IsChecked property.
                    decimal? value = null;
                    for (int i = 0; i < this.UnitDetailList.Count; ++i)
                    {
                        if (i == 0)
                        {
                            value = _units[0].Tier1Deselected;

                        }
                        else if (value != _units[i].Tier1Deselected)
                        {
                            value = null;
                            break;
                        }
                    }

                    return value;

                }
                catch (Exception ex) { Logger.LogException(LogSourceType.ServerApplication, ex); }

                return null;
            }
            set
            {
                if (value == null)
                {
                    return;
                }
                else
                {
                    if (SpinningReserveDislpayHelper.ExpandedResourcesList != null)
                    {
                        // Update the master list of units based on the expanded selection.

                        foreach (ASEngine.BusinessObjects.Entities.UnitDetail expunit in SpinningReserveDislpayHelper.ExpandedResourcesList)
                        {
                            expunit.Tier1Deselected = value.Value;

                            if (this.UnitDetailList.Contains(expunit))
                            {
                                this.UnitDetailList.Remove(expunit);
                                this.UnitDetailList.Add(expunit);
                            }

                        }
                    }
                    else
                    {
                        foreach (ASEngine.BusinessObjects.Entities.UnitDetail unit in this.UnitDetailList)
                        {
                            unit.Tier1Deselected = value.Value;
                        }
                    }
                }
            }
        }

        void unit_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == null)
            {
                Console.Write("Property Not Defined");
            }
        }




        public decimal? AllT2UnitsAreChecked
        {
            get
            {
                try
                {
                    // Determine if all units have the same 
                    // value for the IsChecked property.
                    decimal? value = null;
                    for (int i = 0; i < this.UnitDetailList.Count; ++i)
                    {
                        if (i == 0)
                        {
                            value = _units[0].Tier2Deselected;
                        }
                        else if (value != _units[i].Tier2Deselected)
                        {
                            value = null;
                            break;
                        }
                    }

                    return value;
                }
                catch (Exception ex) { Logger.LogException(LogSourceType.ServerApplication, ex); }

                return null;
            }
            set
            {
                if (value == null)
                {
                    return;
                }
                else
                {
                    if (ExpandedResourcesList != null)
                    {
                        foreach (ASEngine.BusinessObjects.Entities.UnitDetail expunit in SpinningReserveDislpayHelper.ExpandedResourcesList)
                        {
                            expunit.Tier2Deselected = value.Value;

                            if (this.UnitDetailList.Contains(expunit))
                            {
                                this.UnitDetailList.Remove(expunit);
                                this.UnitDetailList.Add(expunit);
                            }
                        }
                    }
                    else
                    {
                        foreach (ASEngine.BusinessObjects.Entities.UnitDetail unit in this.UnitDetailList)
                        {
                            unit.Tier2Deselected = value.Value;
                        }
                    }
                }
            }
        }

        // AllRegDeSelectAreChecked

        public decimal? AllRegDeSelectAreChecked
        {
            get
            {
                try
                {
                    // Determine if all units have the same 
                    // value for the IsChecked property.
                    decimal? value = null;
                    for (int i = 0; i < this.UnitDetailList.Count; ++i)
                    {
                        if (i == 0)
                        {
                            value = _units[0].RegDeselected;
                        }
                        else if (value != _units[i].RegDeselected)
                        {
                            value = null;
                            break;
                        }
                    }

                    return value;
                }
                catch (Exception ex) { Logger.LogException(LogSourceType.ServerApplication, ex); }

                return null;
            }
            set
            {
                if (value == null)
                {
                    return;
                }
                else
                {
                    if (ExpandedResourcesList != null)
                    {
                        foreach (ASEngine.BusinessObjects.Entities.UnitDetail expunit in SpinningReserveDislpayHelper.ExpandedResourcesList)
                        {
                            expunit.RegDeselected = value.Value;

                            if (this.UnitDetailList.Contains(expunit))
                            {
                                this.UnitDetailList.Remove(expunit);
                                this.UnitDetailList.Add(expunit);
                            }
                        }
                    }
                    else
                    {
                        foreach (ASEngine.BusinessObjects.Entities.UnitDetail unit in this.UnitDetailList)
                        {
                            unit.RegDeselected = value.Value;
                        }
                    }
                }
            }
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        void OnPropertyChanged(string prop)
        {
            // Need to re-subscribe to event to maintain handle to ProperChanged
            //this.PropertyChanged += new PropertyChangedEventHandler(unit_PropertyChanged);

            if (this.PropertyChanged == null)
            {
                this.PropertyChanged += new PropertyChangedEventHandler(unit_PropertyChanged);
            }
            else
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(prop));

            }
        }

        #endregion // INotifyPropertyChanged Members

        public void UpdateBO(ASEngine.BusinessObjects.Entities.UnitDetail item)
        {
            SaveController.Instance.AddUpdate(typeof(ASEngine.BusinessObjects.Entities.UnitDetails), item);

        }       


    }  // End Class SpinningReserve   
        
        
        public class ListSRUnitList : List<SpinningReserveDislpayHelper>
        {
            public ListSRUnitList()
            {

                SpinningReserveDislpayHelper l;

                // Load Data Structure
                foreach (ASEngine.BusinessObjects.Entities.UnitDetails unit in SpinningReserveService.unitCache)
                {
                    Add(l = new SpinningReserveDislpayHelper(unit, "ParentSource"));                    
                }



            }

            public ListSRUnitList(decimal caseId)
            {
                SpinningReserveDislpayHelper l;


                // Load Data Structure
                foreach (ASEngine.BusinessObjects.Entities.UnitDetails unit in SpinningReserveService.unitCache)
                {
                    Add(l = new SpinningReserveDislpayHelper(unit, "ParentSource"));         
                }
            }
        }

        public class SRAreaDislpayHelper : INotifyPropertyChanged
        {
            
            ASEngine.BusinessObjects.Entities.AreaZones _areas;
            public ASEngine.BusinessObjects.Entities.AreaZones AreaList { get { return _areas; } }


            public SRAreaDislpayHelper(ASEngine.BusinessObjects.Entities.AreaZones areas)
            {   
                 _areas = areas;


                if (AreaList != null)
                {                   
                    foreach ( ASEngine.BusinessObjects.Entities.AreaZone area in AreaList)
                        area.PropertyChanged += (sender, e) =>
                    {
                            //get modified row.
                        ASEngine.BusinessObjects.Entities.AreaZone o = (ASEngine.BusinessObjects.Entities.AreaZone)sender;
                           o.IsDirty = true;
                            //send update.
                           UpdateBO(o);
                          
                    };            

                }
            
        }


            void area_PropertyChanged(object sender, PropertyChangedEventArgs e)
            {
                if (e.PropertyName == null)
                {
                    Console.Write("Property Not Defined in Area");
                }
            }

           

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        void OnPropertyChanged(string prop)
        {
            if (this.PropertyChanged == null)
            {
                this.PropertyChanged += new PropertyChangedEventHandler(area_PropertyChanged);
            } 
            else
            {
                 this.PropertyChanged(this, new PropertyChangedEventArgs(prop));
               
            }
        }

        #endregion 

        public void UpdateBO(ASEngine.BusinessObjects.Entities.AreaZone item)
            {
                SaveController.Instance.AddUpdate(typeof(ASEngine.BusinessObjects.Entities.AreaZones), item);
                
            }
        }
        
        
        public class ListSRAreaList : List<SRAreaDislpayHelper>
        {
            public ListSRAreaList(decimal caseId)
            {
                SRAreaDislpayHelper areaHelper;
                ASEngine.BusinessObjects.Entities.AreaZones areas = SpinningReserveService.zoneCache.TryGetByKey(caseId);
                Add(areaHelper = new SRAreaDislpayHelper(areas));
            }
        }

        public class ListExpandedDeselectedResourcesList : List<SpinningReserveDislpayHelper>
        {

            public ListExpandedDeselectedResourcesList(ASEngine.BusinessObjects.Entities.UnitDetails resources)
            {
                SpinningReserveDislpayHelper resourcesHelper;

                // Just add the Collection as it will be new each time
                Add(resourcesHelper = new SpinningReserveDislpayHelper(resources, "ExpandedSource"));

            }
        }

     }  // End Namespace
       
       

    

